'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useScrap = undefined;

var _lodash = require('lodash.flatten');

var _lodash2 = _interopRequireDefault(_lodash);

var _scrapDansTesMains = require('scrap-dans-tes-mains');

var _transactionsReduxNormalizer = require('transactions-redux-normalizer');

var _transactionsToolsIsomorphic = require('transactions-tools-isomorphic');

var _parse = require('../tasks/parse');

var _recover = require('../tasks/recover');

var _webshot = require('../tasks/webshot');

var _s = require('../utils/s3');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var useScrap = exports.useScrap = function useScrap(app, config) {
  // unpack
  var bucketName = config.bucketName,
      definition = config.definition,
      description = config.description,
      getMiddlewares = config.getMiddlewares,
      pythonUrl = config.pythonUrl,
      routePath = config.routePath;
  // normalizer

  var appSchema = config.appSchema;
  if (!appSchema) {
    if (definition) {
      appSchema = new _transactionsToolsIsomorphic.SingleSchema('app');
      appSchema.define(definition);
    } else if (description) {
      appSchema = (0, _transactionsReduxNormalizer.createAppSchema)(description);
    }
  }
  // update
  var s3 = (0, _s.getS3)(config);
  // get
  app.get.apply(app, [routePath + '/parse', function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(req, res, next) {
      var _req$query, collectionName, url, _ref2, patch;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              // unpack
              _req$query = req.query, collectionName = _req$query.collectionName, url = _req$query.url;
              // try

              _context.prev = 1;
              _context.next = 4;
              return (0, _parse.parseData)(url, { appSchema: appSchema,
                bucketName: bucketName,
                collectionName: collectionName,
                pythonUrl: pythonUrl,
                s3: s3
              });

            case 4:
              _ref2 = _context.sent;
              patch = _ref2.patch;
              _context.t0 = patch;

              if (!_context.t0) {
                _context.next = 11;
                break;
              }

              _context.next = 10;
              return (0, _recover.recoverData)(patch, { appSchema: appSchema });

            case 10:
              _context.t0 = _context.sent;

            case 11:
              req.body.options = _context.t0;

              // console.log('req.body.options', tinify(req.body.options))
              req.patch = patch;
              _context.next = 19;
              break;

            case 15:
              _context.prev = 15;
              _context.t1 = _context['catch'](1);

              console.warn('scrap api', _context.t1);
              res.json({ error: _context.t1 });

            case 19:
              // next
              req.isNext = true;
              next();

            case 21:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined, [[1, 15]]);
    }));

    return function (_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  }()].concat(_toConsumableArray(getMiddlewares), [function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(_ref4, res, next) {
      var options = _ref4.body.options,
          collectionName = _ref4.query.collectionName,
          data = _ref4.data,
          patch = _ref4.patch;
      var parseDatum, parseEntity, parseDocument;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              // find the document corresponding to the patched entity
              parseDatum = data.find(function (datum) {
                return datum.collectionName === collectionName;
              });
              parseEntity = patch[collectionName + 'ById'][_transactionsToolsIsomorphic.SCRAP];
              // console.log('data', tinify({ data }))
              // console.log('parseDatum', tinify(parseDatum), tinify(parseEntity))

              parseDocument = parseDatum.documents.find(function (_ref5) {
                var id = _ref5.id;
                return parseEntity.id;
              });
              // if it is not found it means we need to add it, it is a new document
              // in the db potentially

              if (!parseDocument) {
                // now we need to make the resolution
                // between naive scrap id of data
                // and data already stored on the db
                // console.log('RESOLUTION', tinify(parseEntity), tinify({ data }), tinify(options))
                options.forEach(function (option, index) {
                  if (option.id === _transactionsToolsIsomorphic.SCRAP) {
                    return;
                  }
                  Object.keys(parseEntity).forEach(function (parseKey) {
                    // replace
                    // console.log('option', option)
                    if (parseEntity[parseKey] === option.id) {
                      // console.log('data[index]', data[index])
                      var _parseDocument = data[index].documents[0];
                      if (!_parseDocument) {
                        console.warn('\n                    parseDocument is null!!! but it shouldnt be for ' + parseKey + ' in ' + collectionName + '\n                  ');
                      }
                      parseEntity[parseKey] = _parseDocument && _parseDocument.id;
                    }
                  });
                });
                // push
                parseDatum.documents.push(parseEntity);
              } else {
                // if not then merge data
                parseDocument = Object.assign(parseDocument, parseEntity);
              }
              // res
              res.json({ data: data });

            case 5:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    }));

    return function (_x4, _x5, _x6) {
      return _ref3.apply(this, arguments);
    };
  }()]));
  // exists
  app.get(routePath + '/exists', function () {
    var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(req, res) {
      var _req$query2, type, url;

      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              // unpack
              _req$query2 = req.query, type = _req$query2.type, url = _req$query2.url;
              // fetch

              _context3.prev = 1;
              _context3.t0 = res;
              _context3.next = 5;
              return (0, _scrapDansTesMains.exists)(url, { type: type });

            case 5:
              _context3.t1 = _context3.sent;

              _context3.t0.json.call(_context3.t0, _context3.t1);

              _context3.next = 13;
              break;

            case 9:
              _context3.prev = 9;
              _context3.t2 = _context3['catch'](1);

              console.warn('exists api', _context3.t2);
              res.json({ error: _context3.t2, isExistingUrl: false });

            case 13:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined, [[1, 9]]);
    }));

    return function (_x7, _x8) {
      return _ref6.apply(this, arguments);
    };
  }());
  // webshot
  app.get(routePath + '/webshot', function () {
    var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(req, res) {
      var url, json;
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              /// unpack
              url = req.query.url;
              // json

              json = {};
              // try

              _context4.prev = 2;
              _context4.t0 = Object;
              _context4.t1 = json;
              _context4.next = 7;
              return (0, _webshot.webshotData)(url, { bucketName: bucketName,
                s3: s3
              });

            case 7:
              _context4.t2 = _context4.sent;

              _context4.t0.assign.call(_context4.t0, _context4.t1, _context4.t2);

              _context4.next = 15;
              break;

            case 11:
              _context4.prev = 11;
              _context4.t3 = _context4['catch'](2);

              json.error = _context4.t3;
              console.warn('webshot api', _context4.t3);

            case 15:
              // send
              res.json(json);

            case 16:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined, [[2, 11]]);
    }));

    return function (_x9, _x10) {
      return _ref7.apply(this, arguments);
    };
  }());
};