'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createAuthorizationFromDir = createAuthorizationFromDir;

var _nodeDir = require('node-dir');

var _nodeDir2 = _interopRequireDefault(_nodeDir);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _transactionsExpressRestMongodb = require('transactions-express-rest-mongodb');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createAuthorizationFromDir(dir) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  // unpack
  var description = config.description,
      isAllDeepJoins = config.isAllDeepJoins,
      isAllJoins = config.isAllJoins;
  // files

  var files = _nodeDir2.default.files(dir, { sync: true });
  // modeNames
  var modeNames = files.filter(function (file) {
    return !/(.*)index.js/.test(file);
  }).map(function (file) {
    return file.split('modes/').slice(-1)[0].replace('.js', '');
  });
  var modeCollectionNames = modeNames.map(function (modeName) {
    return (0, _pluralize2.default)(modeName, 2);
  });
  var modes = modeNames.map(function (modeName) {
    var moduleDir = _path2.default.join(dir, modeName);
    var mode = require(moduleDir).default;
    if (!mode) {
      console.warn('this ' + modeName + ' mode is not well defined');
    } else {
      mode.name = modeName;
    }
    return mode;
  }).filter(function (mode) {
    return mode;
  });
  // authorization
  var authorization = (0, _transactionsExpressRestMongodb.getAuthorizedInitialState)(modes, { description: description,
    modeCollectionNames: modeCollectionNames,
    isAllDeepJoins: isAllDeepJoins,
    isAllJoins: isAllJoins
  });
  // return
  return authorization;
}